/* timecode.c
 * Routines to work with various types of timecode
 *
 * Author: Jason Howard
 * Copyright (c) 2002 - SpectSoft
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "timecode.h"



/* Frame number to timecode */
/* Returns: 1 (Sucessful), 0 (Unsucessful) */
unsigned int tc_frame_to_timecode (unsigned int frame_number, tc_s *timecode) {

	unsigned int df_flag;
	unsigned int i, df_frames;
	unsigned int hour_int, min_int, sec_int, frame_int;

	switch (timecode->type) {

		case TC_TYPE_30:
			frame_int = 30;
			df_flag = 0;
			break;
			
		case TC_TYPE_25:
			frame_int = 25;
			df_flag = 0;
			break;

		case TC_TYPE_24:
			frame_int = 24;
			df_flag = 0;
			break;

		case TC_TYPE_60:
			frame_int = 60;
			df_flag = 0;
			break;
			
		case TC_TYPE_30DF:
			frame_int = 30;
			df_flag = 1;
			break;

		default:
			return 0;
			break;
	}
		
	hour_int = 60*60*frame_int;
	min_int = 60*frame_int;
	sec_int = frame_int;

	/* Check to see if we are dealing with df timecode or not */

				
	timecode->hour = frame_number / hour_int;
	frame_number = frame_number % hour_int;

	timecode->min = frame_number / min_int;
	frame_number = frame_number % min_int;

	timecode->sec = frame_number / sec_int;
	timecode->frame = frame_number % sec_int;

	if (df_flag) {
		
		df_frames = (timecode->hour * 108) + ((timecode->min - (timecode->min / 10)) * 2);

		for (i = 0; i < df_frames; i++) {
			
			timecode->frame ++;
			if (timecode->frame == 30) {
				timecode->frame = 0;
				timecode->sec ++;
			}
			if (timecode->sec == 60) {
				timecode->sec = 0;
				timecode->min ++;
			}
			if (timecode->min == 60) {
				timecode->min = 0;
				timecode->hour ++;
			}
			if (((timecode->min % 10) != 0) && (timecode->sec == 0) &&
					(timecode->frame == 0)) timecode->frame = 2;

		}
			
				
	}
	return 1;
}



/* Time Code to frame number */
/* Returns: Frame number */
unsigned int tc_timecode_to_frame (tc_s *timecode) {

	unsigned int frame_int, hour_int, min_int, sec_int;
	unsigned int df_flag;
	unsigned int frame_number;
	
	switch (timecode->type) {

		case TC_TYPE_30:
			frame_int = 30;
			df_flag = 0;
			break;
			
		case TC_TYPE_25:
			frame_int = 25;
			df_flag = 0;
			break;

		case TC_TYPE_24:
			frame_int = 24;
			df_flag = 0;
			break;

		case TC_TYPE_60:
			frame_int = 60;
			df_flag = 0;
			break;
			
		case TC_TYPE_30DF:
			frame_int = 30;
			df_flag = 1;
			break;

		default:
			return 0;
			break;
	}

	hour_int = 60*60*frame_int;
	min_int = 60*frame_int;
	sec_int = frame_int;
		
	frame_number = (timecode->hour * hour_int) + (timecode->min * min_int) + (timecode->sec * sec_int) + timecode->frame;
	
	/* If this is drop frame timecode make changes to the frame number */	
	if (df_flag) {

		frame_number -= (timecode->hour * 108);
	       	frame_number -= ((timecode->min - (timecode->min / 10)) * 2);

	}
	return frame_number;
		
}


/* Changed a timecode struct into a (human readable) string */
/* Returns: 1 (Successful) */
unsigned int tc_timecode_to_string(tc_s *timecode, char *timecode_string) {

	sprintf(timecode_string, "%03u:%02u:%02u:%02u", timecode->hour, timecode->min, timecode->sec, timecode->frame);
	
	return 1;
}


/* Converts a timecode string into a timecode struct */
/* Returns: 1 (Successful) */
unsigned int tc_string_to_timecode(char *timecode_string, tc_s *timecode) {

	unsigned int h,m,s,f;
	
	sscanf(timecode_string, "%u:%u:%u:%u", &h, &m, &s, &f);
	
	timecode->hour = (unsigned char)h;
	timecode->min = (unsigned char)m;
	timecode->sec = (unsigned char)s;
	timecode->frame = (unsigned char)f;

	
	return 1;
}


/* Converts a timecode type to a human readable string */
/* Returns 1 (Successful) 0 (Unsuccessful) */
unsigned int tc_type_to_string(unsigned int tc_type, char *type) {


	switch (tc_type) {

		case TC_TYPE_24:
			sprintf(type, "24");
			return 1;

		case TC_TYPE_25:
			sprintf(type, "25");
			return 1;

		case TC_TYPE_30:
			sprintf(type, "30");
			return 1;

		case TC_TYPE_60:
			sprintf(type, "60");
			return 1;

		case TC_TYPE_30DF:
			sprintf(type, "30DF");
			return 1;

		default:
			sprintf(type, "ERR");
			return 0;
		}

	/* We shouldn't get here */
	return 0;
}


/* Converts a string into a timecode type */
/* Returns: a timecode type */
unsigned int tc_string_to_type(char *type) {

	if (!strcmp(type, "24"))
		return TC_TYPE_24;

	else if (!strcmp(type, "25"))
		return TC_TYPE_25;

	else if (!strcmp(type, "30"))
		return TC_TYPE_30;

	else if (!strcmp(type, "60"))
		return TC_TYPE_60;

	else if (!strcmp(type, "30DF"))
		return TC_TYPE_30DF;

	else
		return TC_TYPE_ERR;


	/* Shouldn't get here */
	return TC_TYPE_ERR;
}


/* Changes a frame into a string */
/* Returns: 1(Successful) */
unsigned int tc_frame_to_string (unsigned int frame, char *sframe) {

	sprintf(sframe, "%u", frame);

	return 1;
}

/* Changes a string into a frame */
/* Returns: Frame number */
unsigned int tc_string_to_frame (char *sframe) {

	return strtoul(sframe, NULL, 10);
}

